(********************************************************************
 * COPYRIGHT --  
 ********************************************************************
 * Package: SingleAxis
 * File: GlobalSingleAxis.typ
 * Author: liub
 * Created: January 15, 2015
 ********************************************************************
 * Data types of package SingleAxis
 ********************************************************************)

TYPE
	singleAxisStep_Enum : 
		(
		STEP_WAIT_NET_INIT,
		STEP_NETWORK_INIT_ALLOWED,
		STEP_WAIT_NETWORK_INIT_ALLOWED,
		STEP_INIT_STOP,
		SETP_WAIT_INIT_STOP,
		STEP_INIT_SWITCH_OFF,
		STEP_WAIT_INIT_SWITCH_OFF,
		STEP_GLOBAL_INIT,
		STEP_WAIT_GLOBAL_INIT,
		STEP_INIT_HOMING,
		STEP_WAIT_INIT_HOMING,
		STEP_INIT_SWITCH_ON,
		STEP_WAIT_INIT_SWITCH_ON,
		STEP_COMMAND,
		STEP_SWITCH_OFF_CHECK,
		STEP_SWITCH_OFF,
		STEP_WAIT_SWITCH_OFF,
		STEP_SWITCH_ON,
		STEP_WAIT_SWITCH_ON,
		STEP_STOP_MOVE,
		STEP_WAIT_STOP_MOVE,
		STEP_POS_MOVE,
		STEP_NEG_MOVE,
		STEP_POS_JOG,
		STEP_WAIT_POS_JOG,
		STEP_NEG_JOG,
		STEP_WAIT_NEG_JOG,
		STEP_ABS_MOVE,
		STEP_REL_MOVE,
		STEP_HOME,
		STEP_WAIT_HOME,
		STEP_ERROR_STOP,
		STEP_WAIT_ERROR_STOP,
		STEP_ERROR_SWITCH_OFF,
		STEP_WAIT_ERROR_SWITCH_OFF,
		STEP_MESSAGE_TEXT,
		STEP_WAIT_MESSAGE_TEXT,
		STEP_MESSAGE_ACKN
		);
END_TYPE

(**Structure to define servo command*)

TYPE
	singleAxisCmd_typ : 	STRUCT 
		ctrlOn : BOOL;
		ctrlOff : BOOL;
		absMove : BOOL;
		relMove : BOOL;
		posMove : BOOL;
		negMove : BOOL;
		posJog : BOOL;
		negJog : BOOL;
		home : BOOL;
		stop : BOOL;
		errorAcknowledge : BOOL;
	END_STRUCT;
END_TYPE

(**Structure for controller parameter*)

TYPE
	singleAxisPara_typ : 	STRUCT 
		velocity : DINT; (*Units/s*)
		jogVelocity : DINT; (*Units/s*)
		acceleration : DINT; (*Units/s*)
		deceleration : DINT; (*Units/s*)
		relPosition : DINT; (*Units*)
		absPosition : DINT; (*Units*)
		homeMode : DINT;
		homePosition : DINT;
	END_STRUCT;
END_TYPE

(**Structure for monitor servo status*)

TYPE
	singleAxisStatus_typ : 	STRUCT 
		actVelocity : DINT;
		actPosition : DINT;
		inPosition : USINT;
		homeReady : USINT;
		ctrlReady : USINT;
		moveMode : UINT;
		step : singleAxisStep_Enum;
		errorStep : singleAxisStep_Enum;
	END_STRUCT;
END_TYPE

(**Structure for error*)

TYPE
	singleAxisAlarm_typ : 	STRUCT 
		errorNumber : UINT; (*Error message(english)*)
		errorText : ARRAY[0..3]OF STRING[80]; (*Error message(english)*)
	END_STRUCT;
END_TYPE

(**Servo axis control structure*)

TYPE
	singleAxis_typ : 	STRUCT 
		cmd : singleAxisCmd_typ;
		para : singleAxisPara_typ;
		status : singleAxisStatus_typ;
		alarm : singleAxisAlarm_typ;
	END_STRUCT;
END_TYPE
