/********************************************************************
 * COPYRIGHT --  
 ********************************************************************
 * Program: SingleAxis
 * File: SingleAxis.c
 * Author: liub
 * Created: January 15, 2015
 ********************************************************************
 * Implementation of program SingleAxis
 ********************************************************************/

#include <bur/plctypes.h>

#ifdef _DEFAULT_INCLUDES
	#include <AsDefault.h>
#endif

#include <string.h>


/*ʹע⣺*/
/*ÿĿͬ޸
ϵǷswitch on
ֹͣǷswitch off*/
void _INIT SingleAxisINIT(void)
{
    /* Init bit reset */
    initOK = 0;
		
    /* Get AxisObject according to access-name in NC-Deployment-Table */
    ncStatus = ncaccess(ncACP10MAN, SINGLE_AXIS_NAME, &axisObj);
		
    if ( (ncStatus != ncOK) && (ncStatus != 10600) )
    {
        /* The NC object pointer is invalid */
        gSingleAxis.alarm.errorNumber = ncStatus;
        strcpy(gSingleAxis.alarm.errorText[0], "Error to access name in NC-Deployment-Table");
        return;
    }
    /* Get the pointer of NC manager structure */
    pAxisData = (ACP10AXIS_typ*)axisObj;
    if(pAxisData == 0)
    {
        strcpy(gSingleAxis.alarm.errorText[0], "Error to get pAxisData");
        return;
    } 

    if ( pAxisData->size != sizeof(ACP10AXIS_typ) )
    {
        /* The NC data type is not compatible with NC manager */
        gSingleAxis.alarm.errorNumber = pAxisData->size;
        strcpy(gSingleAxis.alarm.errorText[0], "NC data type is not compatible with NC manager");
        return;
    }				
    if ( ( pAxisData->sw_version.nc_manager & 0xFFF0 ) != ( ACP10MAN_H_VERSION & 0xFFF0 ) )
    {
        /* The version is not compatible with NC manager */
        gSingleAxis.alarm.errorNumber = pAxisData->sw_version.nc_manager;
        strcpy(gSingleAxis.alarm.errorText[0], "The version is not compatible with NC manager");
        return;
    }

    pAxisData->message.text.parameter.columns   = sizeof(gSingleAxis.alarm.errorText[0]);
    pAxisData->message.text.parameter.data_len  = sizeof(gSingleAxis.alarm.errorText);
    pAxisData->message.text.parameter.data_adr  = (UDINT)&gSingleAxis.alarm.errorText;
    strcpy((char*)&pAxisData->message.text.parameter.data_modul[0], SINGLE_AXIS_ERROR_TEXT_NAME);

    gSingleAxis.status.errorStep = 0;		/* Reset error step */
	
    step = STEP_WAIT_NET_INIT;
	
    /*˶ʼ*/
    gSingleAxis.para.velocity		= 3000;
    gSingleAxis.para.jogVelocity	= 3000;
    gSingleAxis.para.acceleration	= 10000;
    gSingleAxis.para.deceleration	= 10000;
    gSingleAxis.para.absPosition	= 0;
    gSingleAxis.para.relPosition	= 0;
	
    initOK = 1;
}

void _CYCLIC SingleAxisCYCLIC(void)
{
	/* TODO: Add code here */
    if (initOK == 0)    return;     /*ʼ*/
	
    /* Check for an axis error */		
    if (pAxisData->message.record.number != 0 && gSingleAxis.status.errorStep == 0)
    {		
        gSingleAxis.status.errorStep = step;
        step = STEP_ERROR_STOP;
    }/* if (ncTRUE == pAxisData->monitor.status.error) */
    /* Refresh status data */
    gSingleAxis.alarm.errorNumber   = pAxisData->message.record.number;
    gSingleAxis.status.actPosition	= pAxisData->monitor.s;
    gSingleAxis.status.actVelocity	= pAxisData->monitor.v;
    gSingleAxis.status.moveMode	    = pAxisData->move.mode;
    gSingleAxis.status.ctrlReady	= pAxisData->controller.status;
    gSingleAxis.status.homeReady	= pAxisData->move.homing.status.ok;
    gSingleAxis.status.inPosition	= pAxisData->move.basis.status.in_pos;
    gSingleAxis.status.step		    = step;
    
    /* State sequence for handling the NC Object */
    switch (step)
    {
        /* Wait until the network is initialized */
        case STEP_WAIT_NET_INIT:
            if( pAxisData->network.init == ncTRUE )
            {
                step = STEP_INIT_STOP;
            }
            else if (pAxisData->network.init_allowed == ncTRUE) /*acoposϵϵ-൱ڴacp10cfgexcute net init automatic after acopos reset */
            {
                /*Network initialization allowed*/ 
                step = STEP_NETWORK_INIT_ALLOWED;
            }
            break;
        /* Network initialization */
        case STEP_NETWORK_INIT_ALLOWED:
            ncStatus = ncaction(axisObj, ncNETWORK, ncINIT);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_NETWORK_INIT_ALLOWED;
            }
            break;
        /* Wait network initialization completed */
        case STEP_WAIT_NETWORK_INIT_ALLOWED:
            if (pAxisData->network.init == ncTRUE)
            {
                step = STEP_INIT_STOP;
            }
            break;
        case STEP_INIT_STOP:
            if(pAxisData->move.mode == ncOFF) 
            {
                step = STEP_INIT_SWITCH_OFF;
                break;
            }
            pAxisData->move.stop.index.command = 0;
            ncStatus = ncaction(axisObj,ncMOVE,ncSTOP);
            if(ncStatus == ncOK) 
            {
                step = SETP_WAIT_INIT_STOP;
            }
            break;
        case SETP_WAIT_INIT_STOP:
            if(pAxisData->move.mode == ncOFF)
            {
                step = STEP_INIT_SWITCH_OFF;
            }
            break;
        /* Switch controller off for initialization */
        case STEP_INIT_SWITCH_OFF:
            if (pAxisData->controller.status == ncOFF)
            {
                step = STEP_GLOBAL_INIT;
                break;
            }
            ncStatus = ncaction(axisObj, ncCONTROLLER, ncSWITCH_OFF);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_INIT_SWITCH_OFF;
            }	
            break;
        /* Wait for controller off for initialization */
        case STEP_WAIT_INIT_SWITCH_OFF:
            if (pAxisData->controller.status == ncOFF)
            {
                step = STEP_GLOBAL_INIT;
            }
            break;
        /* Global initialization */
        case STEP_GLOBAL_INIT:
            /*ִйʼִУֹhomingʧ*/
            if(pAxisData->global.init == ncTRUE)    
            {
                step = STEP_INIT_HOMING;
                break;
            }
            
            ncStatus = ncaction(axisObj, ncGLOBAL, ncINIT); 
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_GLOBAL_INIT;
            }               
            break;
        /* Wait for end of global initialization */
        case STEP_WAIT_GLOBAL_INIT:
            if (pAxisData->global.init == ncTRUE)
            {
                step = STEP_INIT_HOMING;
            }
            break;
        /* Start homing procedure ѡǷִһ*/
        case STEP_INIT_HOMING:
            /*homing okִ*/
            if(pAxisData->move.homing.status.ok == ncTRUE)
            {
                step = STEP_COMMAND;
                break;
            }

            pAxisData->move.homing.parameter.mode = ncDIRECT;
            ncStatus = ncaction(axisObj, ncHOMING, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_INIT_HOMING;
            }               
            break;
		
        /* Wait for end of homing procedure */
        case STEP_WAIT_INIT_HOMING:
            if(pAxisData->move.homing.status.ok == ncTRUE)
            {
                step = STEP_INIT_SWITCH_ON;    /*ѡ STEP_COMMAND  STEP_INIT_SWITCH_ON*/
            }
            break;	
        /* Switch controller on */
        case STEP_INIT_SWITCH_ON:
            ncStatus = ncaction(axisObj, ncCONTROLLER, ncSWITCH_ON);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_INIT_SWITCH_ON;
            }
            break;
        /* Wait for controller on */
        case STEP_WAIT_INIT_SWITCH_ON:
            if (pAxisData->controller.status == ncON)
            {
                step = STEP_COMMAND;
            }
            break;	       
        case STEP_COMMAND:
            if (gSingleAxis.cmd.ctrlOff == 1)
            {
                gSingleAxis.cmd.ctrlOff = 0;
                step = STEP_SWITCH_OFF;
            }
            else if (gSingleAxis.cmd.ctrlOn == 1)
            {
                gSingleAxis.cmd.ctrlOn = 0;
                step = STEP_SWITCH_ON;
            }
            else if (gSingleAxis.cmd.stop == 1)
            {
                gSingleAxis.cmd.stop = 0;
                step = STEP_STOP_MOVE;
            }
            else if (gSingleAxis.cmd.posMove == 1)
            {
                gSingleAxis.cmd.posMove = 0;
                step = STEP_POS_MOVE;
            }
            else if (gSingleAxis.cmd.negMove == 1)
            {
                gSingleAxis.cmd.negMove = 0;
                step = STEP_NEG_MOVE;
            }
            else if (gSingleAxis.cmd.posJog == 1)
            {
                step = STEP_POS_JOG;
            }
            else if (gSingleAxis.cmd.negJog == 1)
            {
                step = STEP_NEG_JOG;
            }
            else if (gSingleAxis.cmd.absMove == 1) 
            {
                gSingleAxis.cmd.absMove = 0;
                step = STEP_ABS_MOVE;
            }
            else if (gSingleAxis.cmd.relMove == 1)
            {
                gSingleAxis.cmd.relMove = 0;
                step = STEP_REL_MOVE;
            }
            else if (gSingleAxis.cmd.home == 1)
            {
                gSingleAxis.cmd.home = 0;   
                step = STEP_HOME;   
            }
            break;
        /* Switch controller on */
        case STEP_SWITCH_ON:
            if (pAxisData->controller.status == ncON)
            {
                step = STEP_COMMAND;
                break;
            }
            
            ncStatus = ncaction(axisObj, ncCONTROLLER, ncSWITCH_ON);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_SWITCH_ON;
            }
            break;
        /* Wait for controller on */
        case STEP_WAIT_SWITCH_ON:
            if (pAxisData->controller.status == ncON)
            {
                step = STEP_COMMAND;
            }
            break;	
        /* Start movement in positive direction */
        case STEP_POS_MOVE:
            pAxisData->move.basis.parameter.v_pos = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.a1_pos = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_pos = gSingleAxis.para.deceleration;
            ncStatus = ncaction(axisObj, ncPOS_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_COMMAND;
            }	
            break;
        /* Start movement in negative direction */
        case STEP_NEG_MOVE:
            pAxisData->move.basis.parameter.v_neg = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.a1_neg = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_neg = gSingleAxis.para.deceleration;
            ncStatus = ncaction(axisObj, ncNEG_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_COMMAND;
            }	
            break;
        /* Start jog in positive direction */
        case STEP_POS_JOG:
            pAxisData->move.basis.parameter.v_pos = gSingleAxis.para.jogVelocity;
            pAxisData->move.basis.parameter.a1_pos = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_pos = gSingleAxis.para.deceleration;
            ncStatus = ncaction(axisObj, ncPOS_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_WAIT_POS_JOG;
            }	
            break;
		
        /* Wait for stopping jog in positive direction */
        case STEP_WAIT_POS_JOG:
            /*other cmd reset jog cmd*/
            if(gSingleAxis.cmd.stop == 1 || gSingleAxis.cmd.ctrlOff == 1)
            {
                gSingleAxis.cmd.posJog = 0;
                gSingleAxis.cmd.negJog = 0;
                step = STEP_STOP_MOVE;
                break;
            }
            
            if (gSingleAxis.cmd.posJog == 0)
            {
                step = STEP_STOP_MOVE;
            }
            break;
		
        /* Start jog in negative direction */
        case STEP_NEG_JOG:
            pAxisData->move.basis.parameter.v_neg = gSingleAxis.para.jogVelocity;
            pAxisData->move.basis.parameter.a1_neg = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_neg = gSingleAxis.para.deceleration;
            ncStatus = ncaction(axisObj, ncNEG_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_WAIT_NEG_JOG;
            }	
            break;
		
        /* Wait for stopping jog in negative direction */
        case STEP_WAIT_NEG_JOG:
            /*other cmd reset jog cmd*/
            if(gSingleAxis.cmd.stop == 1 || gSingleAxis.cmd.ctrlOff == 1)
            {
                gSingleAxis.cmd.posJog = 0;
                gSingleAxis.cmd.negJog = 0;
                step = STEP_STOP_MOVE;
                break;
            }
            
            if (gSingleAxis.cmd.negJog == 0)
            {
                step = STEP_STOP_MOVE;
            }
            break;
        /* Start abolute movement */
        case STEP_ABS_MOVE:
            pAxisData->move.basis.parameter.s = gSingleAxis.para.absPosition;
            pAxisData->move.basis.parameter.v_pos = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.v_neg = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.a1_pos = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_pos = gSingleAxis.para.deceleration;
            pAxisData->move.basis.parameter.a1_neg = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_neg = gSingleAxis.para.deceleration;
			
            ncStatus = ncaction(axisObj, ncABS_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_COMMAND;
            }	
            break;
        /* Start relative movement */
        case STEP_REL_MOVE:
            pAxisData->move.basis.parameter.s = gSingleAxis.para.relPosition;
            pAxisData->move.basis.parameter.v_pos = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.v_neg = gSingleAxis.para.velocity;
            pAxisData->move.basis.parameter.a1_pos = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_pos = gSingleAxis.para.deceleration;
            pAxisData->move.basis.parameter.a1_neg = gSingleAxis.para.acceleration;
            pAxisData->move.basis.parameter.a2_neg = gSingleAxis.para.deceleration;
			
            ncStatus = ncaction(axisObj, ncREL_MOVE, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {	
                step = STEP_COMMAND;
            }	
            break;
        /* Start homing procedure */
        case STEP_HOME:
            pAxisData->move.homing.parameter.mode = gSingleAxis.para.homeMode;
            pAxisData->move.homing.parameter.s = gSingleAxis.para.homePosition;
            
            ncStatus = ncaction(axisObj, ncHOMING, ncINIT + ncSTART);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_HOME;
            }
            break;
		
        /* Wait for end of homing procedure */
        case STEP_WAIT_HOME:
            /*other cmd reset jog cmd*/
            if(gSingleAxis.cmd.stop == 1 || gSingleAxis.cmd.ctrlOff == 1)
            {
                gSingleAxis.cmd.posJog = 0;
                gSingleAxis.cmd.negJog = 0;
                step = STEP_STOP_MOVE;
                break;
            }
            
            if (pAxisData->move.homing.status.ok == ncTRUE)
            {
                step = STEP_COMMAND;
            }
            break;	
            
        /* Stop movement */
        case STEP_STOP_MOVE:
            pAxisData->move.stop.index.command = 0;
            ncStatus = ncaction(axisObj, ncMOVE, ncSTOP);
			
            if (ncStatus == ncOK)
            {					
                step = STEP_WAIT_STOP_MOVE;
            }
            if (pAxisData->move.mode == ncOFF)
            {
                step = STEP_COMMAND;
            }
            break;
		
        /* Wait for end of movement */
        case STEP_WAIT_STOP_MOVE:
            if (pAxisData->move.mode == ncOFF)
            {
                step = STEP_COMMAND;
            }	
            break;
		
        /* Wait for end of movement before switch off */
        case STEP_SWITCH_OFF_CHECK:	
            if (pAxisData->move.mode != ncOFF) 
            {	
                pAxisData->move.stop.index.command = 0;				
                ncStatus = ncaction(axisObj, ncMOVE, ncSTOP);
                if (ncStatus == ncOK)
                {
                    step = STEP_SWITCH_OFF;
                }	
            }
            else
            {
                step = STEP_SWITCH_OFF;   
            }	
            break;
		
        /* Switch controller off */
        case STEP_SWITCH_OFF:
            if (pAxisData->move.mode == ncOFF)
            {	
                ncStatus = ncaction(axisObj, ncCONTROLLER, ncSWITCH_OFF);
                if (ncStatus == ncOK)
                {
                    step = STEP_WAIT_SWITCH_OFF;
                }	
            }	
            break;
		
        /* Wait for controller off */
        case STEP_WAIT_SWITCH_OFF:
            if (pAxisData->controller.status == ncOFF)
            {		
                step = STEP_COMMAND;	
            }
            break;
        /* Select movement type */
        case STEP_ERROR_STOP:
            gSingleAxis.cmd.negJog = 0;
            gSingleAxis.cmd.posJog = 0;
            gSingleAxis.cmd.errorAcknowledge = 0;
            /* Stop active movement*/
            if(pAxisData->move.mode != ncOFF)
            {	
                pAxisData->move.stop.index.command = 0;	
                ncStatus = ncaction(axisObj,ncMOVE,ncSTOP);
                if(ncStatus == ncOK)
                {
                    step = STEP_WAIT_ERROR_STOP;
                }	
            }
            else
            {
                step = STEP_ERROR_SWITCH_OFF;   
            }	           
            break;
        case STEP_WAIT_ERROR_STOP:
            if (pAxisData->move.mode == ncOFF)
            {
                step = STEP_ERROR_SWITCH_OFF;
            }
            break;
        /* Switch controller off if required */
        case STEP_ERROR_SWITCH_OFF:
            ncStatus = ncaction(axisObj,ncCONTROLLER,ncSWITCH_OFF);
            if(ncStatus == ncOK)
            {
                step = STEP_WAIT_ERROR_SWITCH_OFF;
            }
            break;
        /* Wait for controller off */
        case STEP_WAIT_ERROR_SWITCH_OFF:
            if(pAxisData->controller.status == ncOFF)
            {
                step = STEP_MESSAGE_TEXT;
            }
            break;
        /* Determine text for current message record */
        case STEP_MESSAGE_TEXT:
            ncStatus = ncaction(axisObj, ncMESSAGE, ncTEXT);
            if (ncStatus == ncOK)
            {
                step = STEP_WAIT_MESSAGE_TEXT;
            }
            break;
		
        /* Wait for end of text determination */
        case STEP_WAIT_MESSAGE_TEXT:
            if ((pAxisData->message.text.status.lines != 0) || 
                (pAxisData->message.text.status.error != 0))
            {
                step = STEP_MESSAGE_ACKN;
            }
            break;
        /* Acknowledge current message record */
        case STEP_MESSAGE_ACKN:
            if(gSingleAxis.cmd.errorAcknowledge == 1)
            {
                gSingleAxis.cmd.errorAcknowledge = 0;
                ncStatus = ncaction(axisObj, ncMESSAGE, ncACKNOWLEDGE);
                if (ncStatus == ncOK)
                {	
                    if (pAxisData->message.record.number == 0)
                    {
                        /* all message records acknowledged */
                        memset(gSingleAxis.alarm.errorText,0,sizeof(gSingleAxis.alarm.errorText));
                        gSingleAxis.status.errorStep = 0;
                        if(pAxisData->global.init == ncTRUE)    
                        {
                            step = STEP_COMMAND;
                        }
                        else
                        {
                            step = STEP_WAIT_NET_INIT;
                        }
                    }
                    else
                    {
                        step = STEP_MESSAGE_TEXT;
                    }
                }                
            }
            break;
    }
}
